"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Item = void 0;
const internal_1 = require("../internal");
class Item extends internal_1.State {
    /**
     * @public
     * Item of Collection
     * @param collection - Collection to which the Item belongs
     * @param data - Data that the Item holds
     * @param config - Config
     */
    constructor(collection, data, config = {}) {
        super(collection.agileInstance(), data, {
            isPlaceholder: config.isPlaceholder,
            key: data[collection.config.primaryKey],
        });
        this.isSelected = false; // If Item is selected by a Selector
        this.collection = () => collection;
        // Reassign Key to assign sideEffects
        this.setKey(data[collection.config.primaryKey]);
    }
    //=========================================================================================================
    // Set Key
    //=========================================================================================================
    /**
     * @internal
     * Updates Key/Name of State
     * @param value - New Key/Name of State
     */
    setKey(value) {
        super.setKey(value);
        if (!value)
            return this;
        // Remove old rebuildGroupsThatIncludeItemKey sideEffect
        this.removeSideEffect(Item.updateGroupSideEffectKey);
        // Add rebuildGroupsThatIncludeItemKey to sideEffects to rebuild Groups that include this Item if it changes
        this.addSideEffect(Item.updateGroupSideEffectKey, (config) => this.collection().rebuildGroupsThatIncludeItemKey(value, config));
        // Initial Rebuild
        this.collection().rebuildGroupsThatIncludeItemKey(value);
        return this;
    }
}
exports.Item = Item;
Item.updateGroupSideEffectKey = "rebuildGroup";
